---
title: "Shiny Intro"
author: "Torben Tvedebrink"
date: "August 23, 2018"
output: 
#  html_document: default
  slidy_presentation: default
runtime: shiny
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(shiny)
library(DT)
library(colourpicker)
```

# What is Shiny?

* R Package for Interactive Web Apps developed by RStudio
* Gives the power of R in a convenient user interface
* Can be written entirely in R

More information: http://shiny.rstudio.com/


<span style="font-size: 0.5em">
*Adapted from Aimee Gott, Mango Solutions, LondonR Workshop, March 2015*
</span>

# A Basic Shiny App

A basic app requires:

* A user interface script, `ui`
* A server script, `server`

Runs using the `runApp` function

#  The User Interface Script

Defines the components of the user interface

  * Page titles
  * Input options
  * Outputs

Defines what the user will see and interact with

# The Server Script

Contains the information to build the app

Requires a call to the function shinyServer

* Contains a function with parameter `input` and `output`

Defines what happens in R

# Examples

## In-line shiny

```{r, echo = FALSE, include=FALSE}
name_ui <- fluidPage(
  titlePanel("Tell me your name!"),
  sidebarLayout(
    sidebarPanel(
      textInput("name", "Enter your name here:")
      ),
    mainPanel(
      textOutput("print_name")
      )
    )
)
```

```{r, echo = FALSE, include=FALSE}
name_server <- function(input, output){
  output$print_name <-
    renderText(paste("Your name is:\n", input$name))
}
```

```{r, echo=FALSE}
shinyApp(ui = name_ui, server = name_server)
```

This was an in-line sort of shiny app that is contained in the `Rmarkdown` document.

## Stand-alone shiny

`ui.R:`
```{r, eval=FALSE}
shinyUI(fluidPage(
  titlePanel("Tell me your name!"),
  sidebarLayout(
    sidebarPanel(
      textInput("name", "Enter your name here:")
      ),
    mainPanel(
      textOutput("print_name")
      )
    )
))
```

`server.R:`
```{r, eval  = FALSE}
shinyServer(function(input, output){
  output$print_name <-
    renderText(paste("Your name is:\n", input$name))
})
```

These two files are saved in a folder `name_app` and envoked by `shiny::runApp('~/name_app')`

# Layouts

* In the stand-alone shiny example we used a `sidebarLayout`
* There are a number of possible layouts (https://shiny.rstudio.com/articles/layout-guide.html)
* Here we will only use the `sidebarLayout`

# `sidebarPanel`

* Define the contents of the sidebar using the `sidebarPanel` function
* Accepts `*Input` functions that specify the app inputs

# `*Input`, `*Output` and `render*` types

There is a number of `*Input` functions:

```{r, echo = FALSE}
inputs <- apropos("^[a-z]*Input$")
inputs[!grepl("roup|update|restore", inputs)]
```

and `*Output`s

```{r, echo = FALSE}
unique(apropos("^[a-z]*Output$"))
```

and `render*` functions

```{r, echo = FALSE}
unique(apropos("^render[A-Z][a-z]*"))
```

# How do these `*Input` appear?

That all takes different types of input types -- this reduces the number of unexpected input your functions receives.

```{r}
checkboxInput(inputId = "my_checkbox", label = "Label for checkboxInput", value = TRUE)
dateInput(inputId = "my_date", label = "Label for dateInput")
dateRangeInput(inputId = "my_dateRange", label = "Label for dateRangeInput")
fileInput(inputId = "my_file", label = "Label for fileInput")
numericInput(inputId = "my_numeric", label = "Label for numericInput", value = pi)
passwordInput(inputId = "my_password", label = "Label for passwordInput")
selectInput(inputId = "my_select", label = "Label for selectInput", choices = c("R", "python"), selected = "R")
selectizeInput(inputId = "my_selectize", label = "Label for selectizeInput", choices = c("R", "python"), selected = "R", 
               multiple = TRUE, options = list(plugins = list("remove_button", "drag_drop")))
sliderInput(inputId = "my_slider", label = "Label for sliderInput", min = 0, max = 10, value = 5)
textAreaInput(inputId = "my_textArea", label = "Label for textAreaInput")
textInput(inputId = "my_text", label = "Label for textInput")
```

# Main Panel

* Define the contents of the main panel using the function mainPanel function
* Can contain outputs using the `*Output` functions
* Can include HTML using a series of functions that replicate the HTML tags

# Defining Outputs

* So far we have just output text
* Shiny also allows us to output graphics, data and images
* We have to define the output in the UI and the Server scripts using different functions

## Outputs

* So far we have just output text
* Shiny also allows us to output graphics, data and images
* We have to define the output in the UI and the Server scripts using different functions

## Rendering outputs

| Output Type | `server.R` function | `ui.R` function   |
|-------------|---------------------|-------------------|
| Text        | `renderPrint`       | `textOutput`      |
| Table       | `renderTable`       | `tableOutput`     |
| Data        | `renderDataTable`   | `dataTableOutput` |
| Plot        | `renderPlot`        | `plotOutput`      |
| Image       | `renderImage`       | `imageOutput`     |

# Example - Render Data

* From the user interface select a dataset from a dropdown menu
* Display the data in a `dataTable`

`ui.R:`
```{r, eval = FALSE}
shinyUI(fluidPage(
  titlePanel("Select your favourite data!"),
  sidebarLayout(
    sidebarPanel(
      selectInput("selectInput", "Select from the dropdown:", choices = c("airquality", "iris", "mtcars"))
      ),
    mainPanel(
      DTOutput("dataOutput")
    )
  )
))
```

`server.R:`
```{r, eval = FALSE}
shinyServer(function(input, output){
  output$dataOutput <-
    renderDT(switch(input$selectInput,
                    "airquality" = airquality,
                    "iris" = iris,
                    "mtcars" = mtcars),
             filter = 'bottom')
})
```

*Note* I used the `DT` library which gives more rich facilities for outputting `dataTables` 
(see more https://rstudio.github.io/DT/shiny.html)

```{r, echo=FALSE}
data_ui <- fluidPage(
  titlePanel("Select your favourite data!"),
  sidebarLayout(
    sidebarPanel(
      selectInput("selectInput", "Select from the dropdown:", choices = c("airquality", "iris", "mtcars"))
      ),
    mainPanel(
      DTOutput("dataOutput")
    )
  )
)

data_server <- function(input, output){
  output$dataOutput <-
    renderDT(switch(input$selectInput,
                    "airquality" = airquality,
                    "iris" = iris,
                    "mtcars" = mtcars),
             filter = 'bottom')
}
```


```{r}
shinyApp(ui = data_ui, server = data_server)
```


# Example - Render plots

* Select a column of the data from a drop down menu
* Plot a histogram of the data

`app.R:`
```{r hist_app, eval=FALSE}
ui <- fluidPage(
  titlePanel("Render plot in a Shiny App"),
  sidebarLayout(
    sidebarPanel(
      selectInput("selectInput", "Select column:", choices = colnames(mtcars))
    ),
    mainPanel(
      plotOutput("plotOutput")
    )
  ))

# Define server logic required to draw a histogram
server <- function(input, output) {
  output$plotOutput <- renderPlot(
    hist(mtcars[,input$selectInput],
         main = paste("Histogram of", input$selectInput),
         xlab = input$selectInput)
  )
}

# Run the application 
shinyApp(ui = ui, server = server)
```

```{r, ref.label="hist_app", echo=FALSE}
```

# Reactivity

## Problem

* Suppose we want to change the colour of a plot in the app, but don't want to re-run everything, e.g.:
  * re-do some demanding calculations
  * simulate a lot of data points
  * reading in a large dataset
  * connecting to a database etc.

## Solution

The `reactive` function
* This lets us create a reactive function
* The function is only called when the relevant inputs are updated
* Our data is only updated when the number of simulations is changed

# Themes and more

There is a rich ecosystem of developers adding functionalities to shiny 
(https://www.r-bloggers.com/the-r-shiny-packages-you-need-for-your-web-apps/). Some add interaction options, other
changes the appearance. The `shinythemes` is one for updating the look rather easily: https://rstudio.github.io/shinythemes/

We simply add `theme = shinytheme("superhero"),` in the `fluidPage()` function to have the "superhero" theme.

# Deploy

You can either host your own RStudio shiny server or use the shinyapps.io to deploy your apps.
See https://shiny.rstudio.com/deploy/ for details.

# Dashboards

There is also a "dashboard" version of Shiny: `shinydashboards`

```{r, message=FALSE}
library(shiny)
library(shinydashboard)

ui <- dashboardPage(
  dashboardHeader(),
  dashboardSidebar(),
  dashboardBody()
)

server <- function(input, output) { }

shinyApp(ui, server)
```

# Examples

```{r}
library(shinydashboard)

ui <- dashboardPage(
  dashboardHeader(title = "Basic dashboard"),
  dashboardSidebar(
     sidebarMenu(
     menuItem("Dashboard", tabName = "dashboard", icon = icon("dashboard")),
     menuItem("Widgets", tabName = "widgets", icon = icon("th"))
    )
  ),
  dashboardBody(
    tabItems(
      # First tab content
      tabItem(tabName = "dashboard",
              fluidRow(
                box(plotOutput("plot1", height = 250)),
                box(
                  title = "Controls",
                  sliderInput("slider", "Number of observations:", 1, 100, 50)
                )
                )
              ),
      tabItem(tabName = "widgets",
        h2("Widgets tab content")
      )
    )
  )
)

server <- function(input, output) {
  set.seed(122)
  histdata <- rnorm(500)

  output$plot1 <- renderPlot({
    data <- histdata[seq_len(input$slider)]
    hist(data)
  })
}

shinyApp(ui, server)
```

But all the technical bits carry over...